"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.runStatement = exports.runStatements = exports.run = void 0;
const utils_1 = require("../utils");
const block_1 = require("./block");
const execution_1 = require("./execution");
const helper_1 = require("./helper");
const path_expression_1 = require("./path-expression");
const MIN_VERSION = 1;
const MAX_VERSION = 2;
async function run(ast, context = {}, extraHelpers) {
    if (ast.version < MIN_VERSION || ast.version > MAX_VERSION) {
        throw new Error(`Unsupported AST version ${ast.version}, parse it again to generate a new AST`);
    }
    const ctx = (0, utils_1.deepCloneNullPrototype)(context);
    const execution = execution_1.Execution.of(ctx, extraHelpers);
    return await runStatements(execution, ast.statements);
}
exports.run = run;
async function runStatements(execution, statements) {
    let result = '';
    for (const statement of statements) {
        const stmtResult = await runStatement(execution, statement);
        if (stmtResult === null || typeof stmtResult === 'undefined') {
            continue;
        }
        if (typeof stmtResult === 'object') {
            result += Object.prototype.toString.call(stmtResult);
            continue;
        }
        result += String(stmtResult);
    }
    return result;
}
exports.runStatements = runStatements;
async function runStatement(execution, statement) {
    switch (statement.type) {
        case 'TEXT':
            return statement.value;
        case 'COMMENT':
            return null;
        case 'LITERAL':
            return statement.value;
        case 'MUSTACHE':
            return await runStatement(execution, statement.expression);
        case 'EXPRESSION':
            return await runExpression(execution, statement);
        case 'BLOCK':
            return await (0, block_1.runBlock)(execution, statement);
        default:
            // TODO track warn unsupported statement type
            return null;
    }
}
exports.runStatement = runStatement;
async function runExpression(execution, expression) {
    if (expression.params.length === 0) {
        return (0, path_expression_1.runPathExpression)(execution, expression);
    }
    return await (0, helper_1.runHelperExpression)(execution, expression);
}
